@@ -1,5 +1,6 @@
 #Python wrapper / library for Einstein Analytics API
 import sys
+import logging
 import browser_cookie3
 import requests
 import json
@@ -16,11 +17,26 @@ import csv
 import unicodecsv
 from unidecode import unidecode
 import math
+import pkg_resources
 
+#init logging
+logging.getLogger(__name__).addHandler(logging.NullHandler())
+logging.basicConfig(format="%(levelname)s: %(message)s")
 
 class salesforceEinsteinAnalytics(object):
-	def __init__(self, env_url, browser):
+	def __init__(self, env_url, browser, suppressVersionWarn=False):
+		self.setLogLvl()
 		self.env_url = env_url
+		
+		#Check if package is current version
+		self.__version__ = '0.2.4'
+		response = requests.get('https://pypi.org/pypi/SalesforceEinsteinAnalytics/json')
+		latest_version = response.json()['info']['version']
+		curr_version = pkg_resources.get_distribution("SalesforceEinsteinAnalytics").version
+		if curr_version != latest_version and suppressVersionWarn==False:
+			logging.warning('New version available. Use pip to upgrade.')
+		
+		#get browser cookie to use in request header
 		try:
 		    if browser == 'chrome':
 		        cj = browser_cookie3.chrome(domain_name=env_url[8:]) #remove first 8 characters since browser cookie does not expect "https://"
@@ -31,15 +47,22 @@ class salesforceEinsteinAnalytics(object):
 		        my_cookies = requests.utils.dict_from_cookiejar(cj)
 		        self.header = {'Authorization': 'Bearer '+my_cookies['sid'], 'Content-Type': 'application/json'}
 		    else:
-		        print('Please select a valid browser (chrome or firefox)')
+		        logging.error('Please select a valid browser (chrome or firefox)')
 		        sys.exit(1)
 		except:
-		    print('ERROR: Could not get session ID.  Make sure you are logged into a live Salesforce session (chrome/firefox).')
+		    logging.error('ERROR: Could not get session ID.  Make sure you are logged into a live Salesforce session (chrome/firefox).')
 		    sys.exit(1)
 
 
-	#set timezone for displayed operation start time
+	def setLogLvl(self, verbose=False):
+		if verbose==True:
+		    logging.getLogger().setLevel(logging.INFO)
+		else:
+		    logging.getLogger().setLevel(logging.WARN)
+
+
 	def get_local_time(self, add_sec=None, timeFORfile=False):
+	    #set timezone for displayed operation start time
 	    curr_time = datetime.datetime.utcnow().replace(tzinfo=tz.tzutc()).astimezone(tz.tzlocal())
 	    if add_sec is not None:
 	        return (curr_time + datetime.timedelta(seconds=add_sec)).strftime("%I:%M:%S %p")
@@ -50,6 +73,9 @@ class salesforceEinsteinAnalytics(object):
 
 
 	def get_dataset_id(self, dataset_name, search_type='API Name', verbose=False):
+		
+		self.setLogLvl(verbose=verbose)
+
 		params = {'pageSize': 50, 'sort': 'Mru', 'hasCurrentOnly': 'true', 'q': dataset_name}
 		dataset_json = requests.get(self.env_url+'/services/data/v46.0/wave/datasets', headers=self.header, params=params) 
 		dataset_df = json_normalize(json.loads(dataset_json.text)['datasets'])
@@ -61,12 +87,11 @@ class salesforceEinsteinAnalytics(object):
 			dataset_df = dataset_df[dataset_df['name'] == dataset_name]
 
 		#show user how many matches that they got.  Might want to use exact API name if getting multiple matches for label search.
-		if verbose == True:
-			print('Found '+str(dataset_df.shape[0])+' matching datasets.')
+		logging.info('Found '+str(dataset_df.shape[0])+' matching datasets.')
 
 		#if dataframe is empty then return not found message or return the dataset ID
 		if dataset_df.empty == True:
-			print('Dataset not found.  Please check name or API name in Einstein Analytics.')
+			logging.warning('Dataset not found.  Please check name or API name in Einstein Analytics.')
 			sys.exit(1)
 		else:
 			dsnm = dataset_df['name'].tolist()[0]
@@ -86,10 +111,11 @@ class salesforceEinsteinAnalytics(object):
 			load statements must have the appropreate spaces: =_load_\"datasetname\";
 		'''
 		
-		if verbose == True:
-			start = time.time()
-			print('Checking SAQL and Finding Dataset IDs...')
-			print('Process started at: '+str(self.get_local_time()))
+		self.setLogLvl(verbose=verbose)
+
+		start = time.time()
+		logging.info('Checking SAQL and Finding Dataset IDs...')
+		logging.info('Process started at: '+str(self.get_local_time()))
 		
 		saql = saql.replace('\"','\\"') #convert UI saql query to JSON format
 		
@@ -108,8 +134,8 @@ class salesforceEinsteinAnalytics(object):
 			saql = saql.replace(load_stmt_old[i], load_stmt_new[i])
 		saql = saql.replace('\\"','\"')
 
-		if verbose == True:
-			print('Running SAQL Query...')
+		
+		logging.info('Running SAQL Query...')
 
 		#run query and return dataframe or save as csv
 		payload = {"query":saql}
@@ -117,22 +143,17 @@ class salesforceEinsteinAnalytics(object):
 		df = json_normalize(json.loads(r.text)['results']['records'])
 		
 		
-		if save_path is not None:
-			if verbose == True:
-				print('Saving result to CSV...')
-			
+		if save_path is not None:			
+			logging.info('Saving result to CSV...')		
 			df.to_csv(save_path, index=False)
-			
-			if verbose == True:
-				end = time.time()
-				print('Dataframe saved to CSV...')
-				print('Completed in '+str(round(end-start,3))+'sec')
+			end = time.time()
+			logging.info('Dataframe saved to CSV...')
+			logging.info('Completed in '+str(round(end-start,3))+'sec')
 			return df
 
 		else:
-			if verbose == True:
-				end = time.time()
-				print('Completed in '+str(round(end-start,3))+'sec')
+			end = time.time()
+			logging.info('Completed in '+str(round(end-start,3))+'sec')
 			return df
 
 
@@ -159,14 +180,14 @@ class salesforceEinsteinAnalytics(object):
 			return history_df
 		
 
-
 	def get_app_user_list(self, app_id=None, save_path=None, verbose=False, max_request_attempts=3):
 		
-		if verbose == True:
-			start = time.time()
-			progress_counter = 0
-			print('Getting app user list and access details...')
-			print('Process started at: '+str(self.get_local_time()))
+		self.setLogLvl(verbose=verbose)
+		
+		start = time.time()
+		progress_counter = 0
+		logging.info('Getting app user list and access details...')
+		logging.info('Process started at: '+str(self.get_local_time()))
 
 		if app_id is None:
 			'''ALERT: CURRENTLY GETTING AN ERROR FOR ALL APP REQUEST
@@ -185,9 +206,8 @@ class salesforceEinsteinAnalytics(object):
 					break
 				except:
 					attempts += 1
-					if verbose == True:
-						print("Unexpected error:", sys.exc_info()[0])
-						print("Trying again...")
+					logging.warning("Unexpected error:", sys.exc_info()[0])
+					logging.warning("Trying again...")
 
 			for app in response['folders']:
 				attempts = 0
@@ -206,16 +226,14 @@ class salesforceEinsteinAnalytics(object):
 						break
 					except:
 						attempts += 1
-						if verbose == True:
-							print("Unexpected error:", sys.exc_info()[0])
-							print("Trying again...")
+						logging.warning("Unexpected error:", sys.exc_info()[0])
+						logging.warning("Trying again...")
 
 			#continue to pull data from next page
 			attempts = 0 # reset attempts for additional pages
 			while next_page is not None:
-				if verbose == True:
-					progress_counter += 25
-					print('Progress: '+str(round(progress_counter/total_size*100,1))+'%')
+				progress_counter += 25
+				logging.info('Progress: '+str(round(progress_counter/total_size*100,1))+'%')
 
 				while attempts < max_request_attempts:
 					try:
@@ -225,13 +243,12 @@ class salesforceEinsteinAnalytics(object):
 						break
 					except KeyError:
 						next_page = None
-						print(sys.exc_info()[0])
+						logging.error(sys.exc_info()[0])
 						break
 					except:
 						attempts += 1
-						if verbose == True:
-							print("Unexpected error:", sys.exc_info()[0])
-							print("Trying again...")
+						logging.warning("Unexpected error:", sys.exc_info()[0])
+						logging.warning("Trying again...")
 
 
 				while attempts < max_request_attempts:
@@ -250,9 +267,8 @@ class salesforceEinsteinAnalytics(object):
 						break
 					except:
 						attempts += 1
-						if verbose == True:
-							print("Unexpected error:", sys.exc_info()[0])
-							print("Trying again...")
+						logging.warning("Unexpected error:", sys.exc_info()[0])
+						logging.warning("Trying again...")
 
 
 		elif app_id is not None:
@@ -270,28 +286,22 @@ class salesforceEinsteinAnalytics(object):
 																"UserType": u['shareType']
 															}, ignore_index=True)
 			else:
-				print('Please input a list or tuple of app Ids')
+				logging.error('Please input a list or tuple of app Ids')
 				sys.exit(1)
 
 		
 		
 		if save_path is not None:
-			if verbose == True:
-				print('Saving result to CSV...')
-
+			logging.info('Saving result to CSV...')
 			app_user_df.to_csv(save_path, index=False)
-			
-			if verbose == True:
-				end = time.time()
-				print('Dataframe saved to CSV...')
-				print('Completed in '+str(round(end-start,3))+'sec')
-
+			end = time.time()
+			logging.info('Dataframe saved to CSV...')
+			logging.info('Completed in '+str(round(end-start,3))+'sec')
 			return app_user_df
 			
 		else: 
-			if verbose == True:
-				end = time.time()
-				print('Completed in '+str(round(end-start,3))+'sec')
+			end = time.time()
+			logging.info('Completed in '+str(round(end-start,3))+'sec')
 			return app_user_df
 
 
@@ -299,10 +309,11 @@ class salesforceEinsteinAnalytics(object):
 		'''
 			update types include:  addNewUsers, fullReplaceAccess, removeUsers, updateUsers
 		'''
-		if verbose == True:
-			start = time.time()
-			print('Updating App Access...')
-			print('Process started at: '+str(self.get_local_time()))
+		self.setLogLvl(verbose=verbose)
+
+		start = time.time()
+		logging.info('Updating App Access...')
+		logging.info('Process started at: '+str(self.get_local_time()))
 		
 		if update_type == 'fullReplaceAccess':
 			shares = user_dict
@@ -375,18 +386,16 @@ class salesforceEinsteinAnalytics(object):
 
 		else:
 			shares = None
-			print('Please choose a user update operation.  Options are: addNewUsers, fullReplaceAccess, removeUsers, updateUsers')
+			logging.error('Please choose a user update operation.  Options are: addNewUsers, fullReplaceAccess, removeUsers, updateUsers')
 			sys.exit(1)
 		
 		if shares is not None:
 			payload = {"shares": shares}
 			r = requests.patch(self.env_url+'/services/data/v46.0/wave/folders/'+app_id, headers=self.header, data=json.dumps(payload))
 
-
-		if verbose == True:
-			end = time.time()
-			print('User Access Updated')
-			print('Completed in '+str(round(end-start,3))+'sec')
+		end = time.time()
+		logging.info('User Access Updated')
+		logging.info('Completed in '+str(round(end-start,3))+'sec')
 
 
 	def update_dashboard_access(self, update_df, update_type, verbose=True):
@@ -394,8 +403,10 @@ class salesforceEinsteinAnalytics(object):
 			Function to make it easier to update access using dashboard names vs finding all apps needed.
 			update dataframe should have the following columns:  Dashboard Id, Access Type, and User Id
 		'''
+		self.setLogLvl(verbose=verbose)
 		pass
 
+
 	def remove_non_ascii(self, df, columns=None):
 		if columns == None:
 			columns = df.columns
@@ -475,17 +486,17 @@ class salesforceEinsteinAnalytics(object):
 		return str(xmd).replace("'",'"')
 
 
-
 	def load_df_to_EA(self, df, dataset_api_name, xmd=None, encoding='UTF-8', operation='Overwrite', useNumericDefaults=True, default_measure_val="0.0", 
 		default_measure_fmt="0.0#", charset="UTF-8", deliminator=",", lineterminator="\r\n", removeNONascii=True, ascii_columns=None, fillna=True, dataset_label=None, verbose=False):
 		'''
 			field names will show up exactly as the column names in the supplied dataframe
 		'''
 
-		if verbose == True:
-			start = time.time()
-			print('Loading Data to Einstein Analytics...')
-			print('Process started at: '+str(self.get_local_time()))
+		self.setLogLvl(verbose=verbose)
+		
+		start = time.time()
+		logging.info('Loading Data to Einstein Analytics...')
+		logging.info('Process started at: '+str(self.get_local_time()))
 
 		dataset_api_name = dataset_api_name.replace(" ","_")
 
@@ -526,12 +537,12 @@ class salesforceEinsteinAnalytics(object):
 		try:
 			json.loads(r1.text)['success'] == True
 		except: 
-			print('ERROR: Upload Config Failed')
-			print(r1.text)
+			logging.error(' Upload Config Failed', exc_info=True)
+			logging.error(r1.text)
 			sys.exit(1)
-		if verbose == True:
-			print('Upload Configuration Complete...')
-			print('Chunking and Uploading Data Parts...')
+		
+		logging.info('Upload Configuration Complete...')
+		logging.info('Chunking and Uploading Data Parts...')
 
 		
 		MAX_FILE_SIZE = 10 * 1000 * 1000 - 49
@@ -551,8 +562,7 @@ class salesforceEinsteinAnalytics(object):
 			range_start += rows_in_part
 			max_data_part += rows_in_part
 			partnum += 1
-			if verbose == True:
-				print('\rChunk '+str(chunk+1)+' of '+str(math.ceil(df_memory / MAX_FILE_SIZE)+1)+' completed', end='', flush=True)
+			logging.info('\rChunk '+str(chunk+1)+' of '+str(math.ceil(df_memory / MAX_FILE_SIZE))+' completed', end='', flush=True)
 
 			payload = {
 				"InsightsExternalDataId" : json.loads(r1.text)['id'],
@@ -564,11 +574,11 @@ class salesforceEinsteinAnalytics(object):
 		try:
 			json.loads(r2.text)['success'] == True
 		except: 
-			print('ERROR: Datapart Upload Failed')
-			print(r2.text)
+			logging.error('\n Datapart Upload Failed', exc_info=True)
+			logging.error(r2.text)
 			sys.exit(1)
-		if verbose == True:
-			print('Datapart Upload Complete...')
+		
+		logging.info('\nDatapart Upload Complete...')
 
 
 		payload = {
@@ -576,12 +586,192 @@ class salesforceEinsteinAnalytics(object):
 				}
 
 		r3 = requests.patch(self.env_url+'/services/data/v46.0/sobjects/InsightsExternalData/'+json.loads(r1.text)['id'], headers=self.header, data=json.dumps(payload))
-		if verbose == True:
-			end = time.time()
-			print('Data Upload Process Started. Check Progress in Data Monitor.')
-			print('Job ID: '+str(json.loads(r1.text)['id']))
-			print('Completed in '+str(round(end-start,3))+'sec')
+		
+		end = time.time()
+		logging.info('Data Upload Process Started. Check Progress in Data Monitor.')
+		logging.info('Job ID: '+str(json.loads(r1.text)['id']))
+		logging.info('Completed in '+str(round(end-start,3))+'sec')
+
+
+	def addArchivePrefix(self, warnList, prefix='[ARCHIVE] ', verbose=False):
+		'''
+		Function to add a warning that an asset will soon be archived.  
+		The name of the dashboard will have the chosen prefix added.
+		
+		max label length is 80 chars and is right trimmed if longer possibly erasing the original title
+
+		Adds prefix to existing label so running twice could overwrite original title
+		'''
+
+		self.setLogLvl(verbose=verbose)
+
+		for asset in warnList:
+			try:
+				r = requests.get(self.env_url+'/services/data/v46.0/wave/dashboards/'+asset, headers=self.header)
+				currentLabel = json.loads(r.text)['label']
+				newLabel = prefix+currentLabel
+				payload = {'label': newLabel[0:79]}
+				r = requests.patch(self.env_url+'/services/data/v46.0/wave/dashboards/'+asset, headers=self.header, data=json.dumps(payload))
+				if json.loads(r.text)['label'] == prefix+currentLabel:
+					logging.info('Successfully updated asset name for: '+asset)
+			except:
+				try:
+					r = requests.get(self.env_url+'/services/data/v46.0/wave/lenses/'+asset, headers=self.header)
+					currentLabel = json.loads(r.text)['label']
+					newLabel = prefix+currentLabel
+					payload = {'label': newLabel[0:79]} #max char len for label = 80
+					r = requests.patch(self.env_url+'/services/data/v46.0/wave/lenses/'+asset, headers=self.header, data=json.dumps(payload))
+					if json.loads(r.text)['label'] == prefix+currentLabel:
+						logging.info('Successfully updated asset name for: '+asset)
+				except:
+					logging.warning(' could not update asset label: '+asset)
+		
+
+	def archiveAssets(self, archiveAppId, ToMoveList, verbose=False):
+		'''
+		ToMoveList can be the Ids for either a dashboard or a lens
+		'''
+		self.setLogLvl(verbose=verbose)
+
+		payload = {'folder': {'id':archiveAppId} }
+
+		for asset in ToMoveList:
+			try:
+				r = requests.patch(self.env_url+'/services/data/v46.0/wave/dashboards/'+asset, headers=self.header, data=json.dumps(payload) )
+				if json.loads(r.text)['folder']['id'] == archiveAppId: #check to ensure response has new folder id
+					logging.info('Successfully archived (type=dashboard): '+asset)
+			except:
+				# if response does not contain the new folder id then try same command for a lens
+				try:
+					r = requests.patch(self.env_url+'/services/data/v46.0/wave/lenses/'+asset, headers=self.header, data=json.dumps(payload) )
+					if json.loads(r.text)['folder']['id'] == archiveAppId: #check to ensure response has new folder id
+						logging.info('Successfully archived (type=lens): '+asset)
+				except:
+					logging.warning(' could not move asset: '+asset)
+
+
+	def getDashboardMetaData(self, appId, max_request_attempts=3, verbose=False):
+		
+		self.setLogLvl(verbose=verbose)
+
+		convertToDateList = ['createdDate','lastModifiedDate','refreshDate']
+		progress_counter = 0
+		assets_df = pd.DataFrame()
+
+		params = {
+				'folderId': appId,
+				'pageSize': 200
+		}
+
+		attempts = 0
+		while attempts < max_request_attempts:
+			try:
+				r1 = requests.get(EA.env_url+'/services/data/v46.0/wave/dashboards', headers=EA.header, params=params)
+				response = json.loads(r1.text)
+				app_assets_df = json_normalize(response['dashboards'])
+				total_size = response['totalSize']
+				next_page = response['nextPageUrl']
+				break
+			except:
+				attempts += 1
+				logging.warning("Unexpected error:", sys.exc_info()[0])
+				logging.warning("Trying again...")
+		for i in convertToDateList:
+			app_assets_df[i].fillna('1900-01-01T00:00:00.000Z', inplace=True)
+			app_assets_df[i] = app_assets_df[i].apply(lambda x: pd.to_datetime(x))
+		assets_df = assets_df.append(app_assets_df, ignore_index=True)
+
+		#continue to pull data from next page if found
+		attempts = 0 # reset attempts for additional pages
+		while next_page is not None:
+			progress_counter += 200
+			logging.info('Progress: '+str(round(progress_counter/total_size*100,1))+'%')
+
+			while attempts < max_request_attempts:
+				try:
+					r1 = requests.get(EA.env_url+next_page, headers=EA.header, params=params)
+					app_assets_df = json_normalize(json.loads(r1.text)['dashboards'])
+					next_page = json.loads(r1.text)['nextPageUrl']
+					break
+				except:
+					attempts += 1
+					logging.warning("Unexpected error:", sys.exc_info()[0])
+					logging.warning("Trying again...")
+			for i in convertToDateList:
+				app_assets_df[i].fillna('1900-01-01T00:00:00.000Z', inplace=True)
+				app_assets_df[i] = app_assets_df[i].apply(lambda x: pd.to_datetime(x))
+			assets_df = assets_df.append(app_assets_df, ignore_index=True)
+		return assets_df
+
+
+	def getLensMetaData(self, appId, max_request_attempts=3, verbose=False):
+		
+		self.setLogLvl(verbose=verbose)
+
+		convertToDateList = ['createdDate','lastModifiedDate','refreshDate']
+		progress_counter = 0
+		assets_df = pd.DataFrame()
+
+		params = {
+				'folderId': appId,
+				'pageSize': 200
+		}
+			
+		attempts = 0
+		while attempts < max_request_attempts:
+			try:
+				r1 = requests.get(EA.env_url+'/services/data/v46.0/wave/lenses', headers=EA.header, params=params)
+				response = json.loads(r1.text)
+				app_assets_df = json_normalize(response['lenses'])
+				total_size = response['totalSize']
+				next_page = response['nextPageUrl']
+				break
+			except:
+				attempts += 1
+				logging.warning("Unexpected error:", sys.exc_info()[0])
+				logging.warning("Trying again...")
+		for i in convertToDateList:
+			app_assets_df[i].fillna('1900-01-01T00:00:00.000Z', inplace=True)
+			app_assets_df[i] = app_assets_df[i].apply(lambda x: pd.to_datetime(x))
+		assets_df = assets_df.append(app_assets_df, ignore_index=True)
+
+		#continue to pull data from next page if found
+		attempts = 0 # reset attempts for additional pages
+		while next_page is not None:
+			progress_counter += 200
+			logging.info('Progress: '+str(round(progress_counter/total_size*100,1))+'%')
+
+			while attempts < max_request_attempts:
+				try:
+					r1 = requests.get(EA.env_url+next_page, headers=EA.header, params=params)
+					app_assets_df = json_normalize(json.loads(r1.text)['lenses'])
+					next_page = json.loads(r1.text)['nextPageUrl']
+					break
+				except:
+					attempts += 1
+					logging.warning("Unexpected error:", sys.exc_info()[0])
+					logging.warning("Trying again...")
+			for i in convertToDateList:
+				app_assets_df[i].fillna('1900-01-01T00:00:00.000Z', inplace=True)
+				app_assets_df[i] = app_assets_df[i].apply(lambda x: pd.to_datetime(x))
+			assets_df = assets_df.append(app_assets_df, ignore_index=True)
+		return assets_df
+
+
+	def getAllAssetMetaData(self, appIdList, max_request_attempts=3, verbose=False):
+		
+		self.setLogLvl(verbose=verbose)
+
+		allAssets_df = pd.DataFrame()
+
+		for app in appIdList:			
+				dashboards_df = self.getDashboardMetaData(appId=app, max_request_attempts=max_request_attempts, verbose=verbose)
+				allAssets_df = allAssets_df.append(dashboards_df, ignore_index=True)				
+				lenses_df = self.getLensMetaData(appId=app, max_request_attempts=max_request_attempts, verbose=verbose)
+				allAssets_df = allAssets_df.append(lenses_df, ignore_index=True)			
+		
+		return allAssets_df
 
 
 if __name__ == '__main__':	
-	pass
+	pass
\ No newline at end of file
